package SOCC::File::Pid;

use strict;
use warnings;
use English;
use Fcntl (':DEFAULT', ':seek', ':flock');

our $VERSION = 0.01;


{
	my $idx = 0;
	use constant FH => $idx++;
	use constant PATH => $idx++;
}


sub new {
	my ($class, $path) = @_;
	
	my ($fh, $self);
	if (sysopen($fh, $path, O_RDWR | O_CREAT)) {
		if (flock($fh, LOCK_EX | LOCK_NB)) {
			# Turn off IO bufferring.
			my $orig_fh = select($fh);
			$OUTPUT_AUTOFLUSH = 1;
			select($orig_fh);
			
			$self = bless([], $class);
			$self->[FH] = $fh;
			$self->[PATH] = $path;
		} else {
			close $fh;
		}
	}
	return $self;
}


sub write {
	my ($self, $pid) = @_;
	
	$pid = $PID unless defined $pid;
	
	my $fh = $self->[FH];
	my $ok = seek($fh, 0, SEEK_SET);
	$ok && ($ok = print($fh "$pid\n"));
	
	return $ok;
}


sub delete {
	my ($self) = @_;
	
	my $ok = unlink($self->[PATH]);
	if (defined fileno $self->[FH]) {
		close $self->[FH];
	}
	return $ok;
}


1;
